package org.zeroturnaround.javarebel.velocity.cbp;

import java.lang.reflect.Modifier;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

public class ClassMapCBP extends JavassistClassBytecodeProcessor {

	public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
		cp.importPackage("org.zeroturnaround.javarebel");

		CtField chacheField = null;
		CtField clazzField = null;

		try {
			chacheField = ctClass.getDeclaredField("methodCache");
			clazzField = ctClass.getDeclaredField("clazz");
		} catch (NotFoundException e) {
			return;
		}
		try {
			CtMethod m = ctClass.getDeclaredMethod("createMethodCache");
			if (!m.getReturnType().equals(chacheField.getType())) {
				return;
			}
		} catch (NotFoundException e) {
			return;
		}
		if (!"java.lang.Class".equals(clazzField.getType().getName())) {
			return;
		}

        ctClass.addInterface(cp.get(ClassEventListener.class.getName()));   

		chacheField.setModifiers(chacheField.getModifiers() & ~Modifier.FINAL);

		CtConstructor[] constructors = ctClass.getConstructors();
		for (int i = 0; i < constructors.length; i++) {
			CtConstructor constructor = constructors[i];
			constructor.insertAfter("ReloaderFactory.getInstance().addClassReloadListener($0);");
		}

	    ctClass.addMethod(CtNewMethod.make(
	            "public int priority() {" +
	            "	return ClassEventListener.PRIORITY_DEFAULT;" +
	            "}",
	    ctClass));

	    CtMethod onClassEventMethod = CtNewMethod.make(
	            "public void onClassEvent(int eventType, Class clazz) {" +
	            "	if (eventType != ClassEventListener.EVENT_RELOADED || clazz != this.clazz) return; " +
	            "	methodCache = createMethodCache();" +
	            "	LoggerFactory.getInstance().log(\"Reloading velocity cache for: \" + clazz);" + 
	            "}",
	    ctClass);
	    ctClass.addMethod(onClassEventMethod);
	}

}
